'''
Autoren: Linus + jl
'''

import serial
import array
import time
import struct
import math
import csv
from typing import cast

class Gesture:
    def __init__(self, name: str, title: str):
        self._title = title
        self._gesture : list[tuple[int, int, int]] = []
        self._captured : list[tuple[int, int, int]] = []
        with open(name+'.csv', 'r') as csvfile:
            reader = csv.reader(csvfile, delimiter=',', quotechar='|')
            for row in reader:
                self._gesture.append((int(row[0]), int(row[1]), int(row[2])))
        self.sample_length = len(self._gesture)
        self.max_distance = math.sqrt(3*self.sample_length*256**2)

    def recognize(self, v : tuple[int, int, int]):
        self._captured.append((v[0], v[1], v[2]))
        if len(self._captured) >= self.sample_length:   
            dissimilarity = 0
            i = 0
            for point in self._captured:
                dx = (point[0] - self._gesture[i][0])**2
                dy = (point[1] - self._gesture[i][1])**2
                dz = (point[2] - self._gesture[i][2])**2
                dissimilarity += dx + dy + dz
                i += 1
            similarity = (1-math.sqrt(dissimilarity)/self.max_distance)*100
            self._captured.pop(0)
            if similarity > 91: 
                print(f"{self._title} erkannt! Ähnlichkeit: {similarity:.3g} %\n")
                self._captured.clear()
                
gestures=[
Gesture('gesture1','Drehung gegen den Uhrzeigersinn'),
Gesture('gesture2','Drehung mit dem Uhrzeigersinn'),
Gesture('gesture3','Bewegung hoch runter'),   
Gesture('gesture4','Bewegung runter hoch'),
Gesture('gesture5','Bewegung rechts links'),
Gesture('gesture6','Bewegung hinter den Kopf'),
Gesture('gesture7','Drehung des Handgelenks'),
Gesture('gesture8','Hand auf Schulter')
]
     
ser = serial.Serial(port = "COM5", baudrate = 115200, timeout = 1) # COM-Port ggf. anpassen!
ser.write(array.array('B', [0xFF, 0x07, 0x03, 0x00, 0x00, 0x00, 0x00]).tobytes())
ser.read(3)

print("Started reading")

for i in range(1500):
    ser.write(array.array('B', [0xFF, 0x08, 0x07, 0x00, 0x00, 0x00, 0x00]).tobytes())
    accel = ser.read(7)
    if len(accel) == 7 and accel[3] == 1:      
        v = cast(tuple[int, int, int], struct.unpack('bbb', accel[4:7])) # cast ist nur für den Typprüfer
        for gesture in gestures:
            gesture.recognize(v)
    time.sleep(0.03)
ser.write(array.array('B', [0xFF, 0x09, 0x03, 0x00, 0x00, 0x00, 0x00]).tobytes())
ser.close()